#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>

#include "gsc16ao12_ioctl.h"

char driver_name[80]="/dev/gsc_16ao12\0";
int pass;

int init_gs (void) ;

#define WORDS_TO_SEND 1024

int main(int argc, char *argv[])
{
    struct register_params rw_q;
    unsigned long buffer[32768], total, count[16];
    int fd, res, i, print, nread;
    int board_type;
    unsigned long param;

    if (argc < 2) {
        printf("%s: missing argument\nusage: testapp [0|1|2|...]\n", argv[0]);
        return (1);
    }

    strcat(driver_name, argv[1]);
    printf("about to open %s\n",driver_name);

    fd = open(driver_name, O_RDWR);
    if (fd < 0) {
        printf("%s: can not open device %s\n", argv[0], driver_name);
        return (1);
    }
    init_keyboard();

    rw_q.eRegister = 0x0;
    rw_q.ulValue = 0x2000;
    res = ioctl(fd, (unsigned long)IOCTL_READ_REGISTER, &rw_q);
    if (res < 0) {
        printf("%s: ioctl IOCTL_READ_REGISTER failed\n", argv[0]);
        goto out;
    }

    printf("before reset: BCR is 0x%lx\n", rw_q.ulValue);


    param=5; // seconds
    res = ioctl(fd, IOCTL_SET_TIMEOUT, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_SET_TIMEOUT failed\n", argv[0]);
        goto out;
    }

    res = ioctl(fd, IOCTL_INIT_BOARD, NULL);
    if (res < 0) {
        printf("%s: ioctl IOCTL_INIT_BOARD failed\n", argv[0]);
        goto out;
    }
    printf("board init OK %d\n",i);

    rw_q.eRegister = BOARD_CTRL_REG;
    res = ioctl(fd, IOCTL_READ_REGISTER, &rw_q);
    if (res < 0) {
        printf("%s: ioctl IOCTL_READ_REGISTER failed\n", argv[0]);
        goto out;
    }
    printf("after reset: BCR is 0x%lx\n", rw_q.ulValue);

    res = ioctl(fd, IOCTL_AUTOCALIBRATE, NULL);
    if (res < 0) {
        printf("%s: ioctl IOCTL_AUTO_CAL failed %d\n", argv[0],i);
        goto out;
    }
    printf("auto calibration OK\n");

    // read autocalibration values - this is a reserved register.

    rw_q.eRegister = 0x14/4;
    res = ioctl(fd, IOCTL_READ_REGISTER, &rw_q);
    if (res < 0) {
        printf("%s: ioctl IOCTL_READ_REGISTER failed\n", argv[0]);
        goto out;
    }
    printf("after reset: Autocalibration values are 0x%lx\n", rw_q.ulValue);
    
    //   set clock for 1 kHz.
    
    //param = 30000;
    param = 300;
    res = ioctl(fd, (unsigned long)IOCTL_PROGRAM_RATE_GEN, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_PROGRAM_RATE_GEN failed\n", argv[0]);
        goto out;
    }

    // select only the first channel

    param = CHAN_0_ENABLE;
    res = ioctl(fd, (unsigned long)IOCTL_CHANNEL_ENABLE, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_CHANNEL_ENABLE failed\n", argv[0]);
        goto out;
    }

    //   two's comp data format.
    
    param = TWOS_COMP;
    res = ioctl(fd, (unsigned long)IOCTL_SELECT_DATA_FORMAT, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_SELECT_DATA_FORMAT failed\n", argv[0]);
        goto out;
    }

    // simultaneous clocking.
    
    param = SIMULTANEOUS;
    res = ioctl(fd, (unsigned long)IOCTL_SELECT_OUT_CLKING_MODE, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_SELECT_OUT_CLKING_MODE failed\n", argv[0]);
        goto out;
    }

    //   continuous or burst output.
   
    param = CONT_MODE;
    //param = BURST_MODE;
    res = ioctl(fd, (unsigned long)IOCTL_SELECT_SAMPLING_MODE, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_SELECT_SAMPLING_MODE failed\n", argv[0]);
        goto out;
    }
    
    //   internal clock.
    
    param = INTERNAL;
    res = ioctl(fd, (unsigned long)IOCTL_SELECT_CLK_SOURCE, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_SELECT_CLK_SOURCE failed\n", argv[0]);
        goto out;
    }

    //   enable clock.
    
    res = ioctl(fd, (unsigned long)IOCTL_ENABLE_CLK); 
    if (res < 0) {
        printf("%s: ioctl IOCTL_ENABLE_CLK failed\n", argv[0]);
        goto out;
    }
    
    //   create a buffer of data.

    for (i=0;i<WORDS_TO_SEND;i++)
    {
        buffer[i]=i*0x100;
    }

    // the end of buffer flag really only means anything if
    // the output buffer is set to 'closed,' which this 
    // program does not do, so this just demonstrates how to
    // set the flag.

    buffer[WORDS_TO_SEND]|=ODB_EOF_FLAG; // mark end of buffer.
    
    // write a buffer with PIO.  The driver 'knows' to wait for
    // the buffer to be ready.
    
    printf("before PIO write\n");

    param = DMA_DISABLE;
    res = ioctl(fd, (unsigned long)IOCTL_SET_DMA_MODE, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_WRITE_MODE_CONFIG failed\n", argv[0]);
        goto out;
    }

    for(i=0;i<5;i++)
    {
        res = write(fd, buffer, WORDS_TO_SEND*4);
        if (res < 0){
            printf("read error -> after write...res = %d\n",res);
            goto out;
        }
    }

    printf("after PIO write\n");
    
    // write buffers with DMA.  The driver 'knows' to wait for 
    // the buffer ready.
    
    param = DMA_ENABLE;
    res = ioctl(fd, (unsigned long)IOCTL_SET_DMA_MODE, &param);
    if (res < 0) {
        printf("%s: ioctl IOCTL_WRITE_MODE_CONFIG failed\n", argv[0]);
        goto out;
    }

    printf("before DMA write loop...press any key to stop...\n");
    pass=0;

    while(!kbhit())
    {
        pass++;
    {
        res = write(fd, buffer, WORDS_TO_SEND*4);
        if (res < 0){
            printf("read error -> after write...res = %d\n",res);
            goto out;
        }
    }
  
//    printf(">>>>>>>End of pass %d  Press any key to stop...\n",pass);
    printf("%d\r",pass);
    }
    
    printf("Closing hardware...\n");
    //   disable clock.

    res = ioctl(fd, (unsigned long)IOCTL_DISABLE_CLK);
    if (res < 0) {
        printf("%s: ioctl IOCTL_DISABLE_CLK failed\n", argv[0]);
        goto out;
    }
out:

    close_keyboard();
    printf("\n");
    close(fd);
    return 0;
}
